package com.startbot.utils;

import java.time.*;
import java.time.format.DateTimeFormatter;
import java.time.temporal.ChronoUnit;
import java.util.Date;
import java.util.Locale;

/**
 * 时间工具类，提供时间戳格式化和常用时间操作方法
 */
public class TimeUtil {

    // 常用时间格式
    public static final String PATTERN_YYYY_MM_DD = "yyyy-MM-dd";
    public static final String PATTERN_YYYY_MM_DD_HH_MM_SS = "yyyy-MM-dd HH:mm:ss";
    public static final String PATTERN_YYYY_MM_DD_HH_MM_SS_SSS = "yyyy-MM-dd HH:mm:ss.SSS";
    public static final String PATTERN_YYYYMMDD = "yyyyMMDD";
    public static final String PATTERN_YYYYMMDDHHMMSS = "yyyyMMddHHmmss";
    public static final String PATTERN_YYYYMMDDHHMMSSSSS = "yyyyMMddHHmmssSSS";
    public static final String PATTERN_HH_MM_SS = "HH:mm:ss";

    // 时区设置
    private static final ZoneId DEFAULT_ZONE = ZoneId.systemDefault();

    /**
     * 将时间戳格式化为指定格式的字符串
     *
     * @param timestamp 时间戳（毫秒）
     * @param pattern   时间格式
     * @return 格式化后的时间字符串
     */
    public static String formatTimestamp(long timestamp, String pattern) {
        LocalDateTime dateTime = LocalDateTime.ofInstant(Instant.ofEpochMilli(timestamp), DEFAULT_ZONE);
        return dateTime.format(DateTimeFormatter.ofPattern(pattern));
    }

    /**
     * 将时间戳格式化为默认格式（yyyy-MM-dd HH:mm:ss）的字符串
     *
     * @param timestamp 时间戳（毫秒）
     * @return 格式化后的时间字符串
     */
    public static String formatTimestamp(long timestamp) {
        return formatTimestamp(timestamp, PATTERN_YYYY_MM_DD_HH_MM_SS);
    }

    /**
     * 将Date对象格式化为指定格式的字符串
     *
     * @param date    Date对象
     * @param pattern 时间格式
     * @return 格式化后的时间字符串
     */
    public static String formatDate(Date date, String pattern) {
        if (date == null) {
            return null;
        }
        return formatTimestamp(date.getTime(), pattern);
    }

    /**
     * 将LocalDateTime对象格式化为指定格式的字符串
     *
     * @param dateTime LocalDateTime对象
     * @param pattern  时间格式
     * @return 格式化后的时间字符串
     */
    public static String formatLocalDateTime(LocalDateTime dateTime, String pattern) {
        if (dateTime == null) {
            return null;
        }
        return dateTime.format(DateTimeFormatter.ofPattern(pattern));
    }

    /**
     * 将字符串解析为LocalDateTime对象
     *
     * @param dateStr 时间字符串
     * @param pattern 时间格式
     * @return LocalDateTime对象
     */
    public static LocalDateTime parseLocalDateTime(String dateStr, String pattern) {
        return LocalDateTime.parse(dateStr, DateTimeFormatter.ofPattern(pattern));
    }

    /**
     * 将字符串解析为时间戳
     *
     * @param dateStr 时间字符串
     * @param pattern 时间格式
     * @return 时间戳（毫秒）
     */
    public static long parseTimestamp(String dateStr, String pattern) {
        LocalDateTime dateTime = parseLocalDateTime(dateStr, pattern);
        return dateTime.atZone(DEFAULT_ZONE).toInstant().toEpochMilli();
    }

    /**
     * 获取当前时间戳（毫秒）
     *
     * @return 当前时间戳
     */
    public static long currentTimestamp() {
        return System.currentTimeMillis();
    }

    /**
     * 获取当前日期（yyyy-MM-dd）
     *
     * @return 当前日期字符串
     */
    public static String currentDate() {
        return LocalDate.now().format(DateTimeFormatter.ofPattern(PATTERN_YYYY_MM_DD));
    }

    /**
     * 获取当前时间（HH:mm:ss）
     *
     * @return 当前时间字符串
     */
    public static String currentTime() {
        return LocalTime.now().format(DateTimeFormatter.ofPattern(PATTERN_HH_MM_SS));
    }

    /**
     * 获取当前日期时间（yyyy-MM-dd HH:mm:ss）
     *
     * @return 当前日期时间字符串
     */
    public static String currentDateTime() {
        return LocalDateTime.now().format(DateTimeFormatter.ofPattern(PATTERN_YYYY_MM_DD_HH_MM_SS));
    }

    /**
     * 获取指定日期的开始时间（00:00:00）
     *
     * @param dateStr 日期字符串（yyyy-MM-dd）
     * @return 开始时间的时间戳
     */
    public static long getStartOfDay(String dateStr) {
        LocalDate date = LocalDate.parse(dateStr, DateTimeFormatter.ofPattern(PATTERN_YYYY_MM_DD));
        return date.atStartOfDay(DEFAULT_ZONE).toInstant().toEpochMilli();
    }

    /**
     * 获取指定日期的结束时间（23:59:59.999）
     *
     * @param dateStr 日期字符串（yyyy-MM-dd）
     * @return 结束时间的时间戳
     */
    public static long getEndOfDay(String dateStr) {
        LocalDate date = LocalDate.parse(dateStr, DateTimeFormatter.ofPattern(PATTERN_YYYY_MM_DD));
        return date.atTime(23, 59, 59, 999_000_000).atZone(DEFAULT_ZONE).toInstant().toEpochMilli();
    }

    /**
     * 获取两个时间戳之间的差值（单位：秒）
     *
     * @param startTime 开始时间戳
     * @param endTime   结束时间戳
     * @return 差值（秒）
     */
    public static long getTimeDiffInSeconds(long startTime, long endTime) {
        return (endTime - startTime) / 1000;
    }

    /**
     * 获取两个时间戳之间的差值（单位：分钟）
     *
     * @param startTime 开始时间戳
     * @param endTime   结束时间戳
     * @return 差值（分钟）
     */
    public static long getTimeDiffInMinutes(long startTime, long endTime) {
        return getTimeDiffInSeconds(startTime, endTime) / 60;
    }

    /**
     * 获取两个时间戳之间的差值（单位：小时）
     *
     * @param startTime 开始时间戳
     * @param endTime   结束时间戳
     * @return 差值（小时）
     */
    public static long getTimeDiffInHours(long startTime, long endTime) {
        return getTimeDiffInMinutes(startTime, endTime) / 60;
    }

    /**
     * 获取两个时间戳之间的差值（单位：天）
     *
     * @param startTime 开始时间戳
     * @param endTime   结束时间戳
     * @return 差值（天）
     */
    public static long getTimeDiffInDays(long startTime, long endTime) {
        return getTimeDiffInHours(startTime, endTime) / 24;
    }

    /**
     * 判断两个时间戳是否在同一天
     *
     * @param timestamp1 时间戳1
     * @param timestamp2 时间戳2
     * @return 是否在同一天
     */
    public static boolean isSameDay(long timestamp1, long timestamp2) {
        LocalDate date1 = Instant.ofEpochMilli(timestamp1).atZone(DEFAULT_ZONE).toLocalDate();
        LocalDate date2 = Instant.ofEpochMilli(timestamp2).atZone(DEFAULT_ZONE).toLocalDate();
        return date1.isEqual(date2);
    }

    /**
     * 获取指定日期偏移指定天数后的日期
     *
     * @param dateStr  日期字符串（yyyy-MM-dd）
     * @param days     偏移天数（正数表示未来，负数表示过去）
     * @param withTime 是否包含时间（true: 返回yyyy-MM-dd HH:mm:ss，false: 返回yyyy-MM-dd）
     * @return 偏移后的日期字符串
     */
    public static String getDateAfter(String dateStr, int days, boolean withTime) {
        LocalDate date = LocalDate.parse(dateStr, DateTimeFormatter.ofPattern(PATTERN_YYYY_MM_DD));
        LocalDate newDate = date.plusDays(days);
        return withTime ?
                newDate.atStartOfDay().format(DateTimeFormatter.ofPattern(PATTERN_YYYY_MM_DD_HH_MM_SS)) :
                newDate.format(DateTimeFormatter.ofPattern(PATTERN_YYYY_MM_DD));
    }

    /**
     * 将时间戳转换为相对时间描述（如：刚刚、1分钟前、1小时前、昨天、3天前）
     *
     * @param timestamp 时间戳（毫秒）
     * @return 相对时间描述
     */
    public static String toRelativeTime(long timestamp) {
        long now = currentTimestamp();
        long diff = now - timestamp;

        if (diff < 60 * 1000) {
            return "刚刚";
        } else if (diff < 60 * 60 * 1000) {
            return diff / (60 * 1000) + "分钟前";
        } else if (isSameDay(timestamp, now)) {
            return diff / (60 * 60 * 1000) + "小时前";
        } else if (diff < 24 * 60 * 60 * 1000) {
            return "昨天";
        } else if (diff < 3 * 24 * 60 * 60 * 1000) {
            return diff / (24 * 60 * 60 * 1000) + "天前";
        } else {
            return formatTimestamp(timestamp, PATTERN_YYYY_MM_DD);
        }
    }

    /**
     * 将Date转换为LocalDateTime
     *
     * @param date Date对象
     * @return LocalDateTime对象
     */
    public static LocalDateTime dateToLocalDateTime(Date date) {
        return date.toInstant().atZone(DEFAULT_ZONE).toLocalDateTime();
    }

    /**
     * 将LocalDateTime转换为Date
     *
     * @param dateTime LocalDateTime对象
     * @return Date对象
     */
    public static Date localDateTimeToDate(LocalDateTime dateTime) {
        return Date.from(dateTime.atZone(DEFAULT_ZONE).toInstant());
    }

    /**
     * 获取本周第一天（周一）的日期
     *
     * @return 本周第一天的日期字符串（yyyy-MM-dd）
     */
    public static String getFirstDayOfWeek() {
        LocalDate now = LocalDate.now();
        LocalDate firstDay = now.with(java.time.temporal.TemporalAdjusters.previousOrSame(DayOfWeek.MONDAY));
        return firstDay.format(DateTimeFormatter.ofPattern(PATTERN_YYYY_MM_DD));
    }

    /**
     * 获取本月第一天的日期
     *
     * @return 本月第一天的日期字符串（yyyy-MM-dd）
     */
    public static String getFirstDayOfMonth() {
        LocalDate now = LocalDate.now();
        LocalDate firstDay = now.with(java.time.temporal.TemporalAdjusters.firstDayOfMonth());
        return firstDay.format(DateTimeFormatter.ofPattern(PATTERN_YYYY_MM_DD));
    }

    /**
     * 判断是否是闰年
     *
     * @param year 年份
     * @return 是否是闰年
     */
    public static boolean isLeapYear(int year) {
        return Year.isLeap(year);
    }

    /**
     * 获取指定日期是星期几
     *
     * @param dateStr 日期字符串（yyyy-MM-dd）
     * @return 星期几（如：星期一、星期日）
     */
    public static String getDayOfWeek(String dateStr) {
        LocalDate date = LocalDate.parse(dateStr, DateTimeFormatter.ofPattern(PATTERN_YYYY_MM_DD));
        DayOfWeek dayOfWeek = date.getDayOfWeek();
        return dayOfWeek.getDisplayName(java.time.format.TextStyle.FULL, Locale.CHINA);
    }

    /**
     * 计算两个日期之间相差的天数
     *
     * @param startDate 开始日期（yyyy-MM-dd）
     * @param endDate   结束日期（yyyy-MM-dd）
     * @return 相差的天数
     */
    public static long daysBetween(String startDate, String endDate) {
        LocalDate start = LocalDate.parse(startDate, DateTimeFormatter.ofPattern(PATTERN_YYYY_MM_DD));
        LocalDate end = LocalDate.parse(endDate, DateTimeFormatter.ofPattern(PATTERN_YYYY_MM_DD));
        return ChronoUnit.DAYS.between(start, end);
    }

    /**
     * 判断一个日期是否在另一个日期之前
     *
     * @param date1 日期1（yyyy-MM-dd）
     * @param date2 日期2（yyyy-MM-dd）
     * @return date1是否在date2之前
     */
    public static boolean isBefore(String date1, String date2) {
        LocalDate d1 = LocalDate.parse(date1, DateTimeFormatter.ofPattern(PATTERN_YYYY_MM_DD));
        LocalDate d2 = LocalDate.parse(date2, DateTimeFormatter.ofPattern(PATTERN_YYYY_MM_DD));
        return d1.isBefore(d2);
    }

    /**
     * 判断一个日期是否在另一个日期之后
     *
     * @param date1 日期1（yyyy-MM-dd）
     * @param date2 日期2（yyyy-MM-dd）
     * @return date1是否在date2之后
     */
    public static boolean isAfter(String date1, String date2) {
        LocalDate d1 = LocalDate.parse(date1, DateTimeFormatter.ofPattern(PATTERN_YYYY_MM_DD));
        LocalDate d2 = LocalDate.parse(date2, DateTimeFormatter.ofPattern(PATTERN_YYYY_MM_DD));
        return d1.isAfter(d2);
    }

    /**
     * 获取指定日期所在月份的天数
     *
     * @param dateStr 日期字符串（yyyy-MM-dd）
     * @return 该月的天数
     */
    public static int getDaysOfMonth(String dateStr) {
        LocalDate date = LocalDate.parse(dateStr, DateTimeFormatter.ofPattern(PATTERN_YYYY_MM_DD));
        return date.lengthOfMonth();
    }
}

